#! /usr/bin/perl
#
#

use strict;
use Getopt::Long;

my $d_threshold = 0.7;
my $window = 0;
my $help = 0;

GetOptions ("threshold=s" => \$d_threshold,
			"window=i" => \$window,
			"help" => \$help);

if($help || $window == 0 || @ARGV == 0)
{
	print "metaCaller.pl --threshold <t> --window <w> <file>\n";
}

my $file = $ARGV[0];


# Build the state map
my %stateMap;
$stateMap{"Z"} = "Z";
$stateMap{"R"} = "R";
$stateMap{"N"} = "N";
$stateMap{"C1"} = "C";
$stateMap{"C2"} = "G";
$stateMap{"C3"} = "G";
$stateMap{"C0.5"} = "L";

my $complexState = "X";

my $chrWindows;

open(FILE, $file);
while(my $line = <FILE>)
{
	chomp $line;
	my @fields = split(/\t/, $line);
	push @{ $chrWindows->{ $fields[0] } }, $line;
}
close(FILE);

foreach my $k (keys %{$chrWindows})
{
	my @elements = @{$chrWindows->{$k}};
	my $index = 0;
	while($index < @elements)
	{
		# Process the array 10 elements at a time
		my $start = $index;
		my $end = $start + $window - 1;
		if($end > (@elements - 1))
		{
			$end = @elements - 1;
		}
		my @buffer = @elements[$start..$end];
		processWindow(\@buffer);
		$index = $end + 1;
	}
}

sub processWindow
{
	my($window) = @_;

	my $start = -1;
	my $end = -1;
	my $chr = "";
	my %counts;
	
	foreach my $l (@{$window})
	{
		my ($label, $coord, $d1, $d2, $d3, $state) = split(/\t/, $l);
		if($coord < $start || $start == -1)
		{
			$start = $coord;	
		}

		if($coord > $end || $end == -1)
		{
			$end = $coord;
		}

		$counts{$stateMap{$state}}++;
		$chr = $label;
	}

	my $sum = 0;
	my $max = 0;
	my $max_k;

	while(my ($key, $value) = each(%counts))
	{
		if($value > $max)
		{
			$max = $value;
			$max_k = $key;
		}

		$sum += $value;
	}

	my $outState;
	if($max / $sum >= $d_threshold)
	{
		$outState = $max_k;
	}
	else
	{
		$outState = $complexState;
	}
	
	# Output the record
	print join("\t", ($chr,$start,$end,$outState)) . "\n";
}

