#! /usr/bin/perl

use strict;
use Getopt::Long;
use POSIX qw(ceil floor);

my $win_chr = "-1";
my $win_idx = 0;
my $win_start = 0;
my $win_size = 1000;
my $last_pos = 1000;
my $win_end = 0;
my $scale = 1000000;
my $num_points = 0;
my $num_covered = 0;
my $formatStr = join("\t", ("%s", "%.6f", "%.4f", "%.4f", "%.3f", "%.3f", "%.3f", "%.3f")) . "\n";

GetOptions("window=i" => \$win_size);

my ($s_depth, $s_het, $s_rms, $n) = initializeWindow("-1", 0);
while(<>)
{
	my @fields = split(/\t/);
	die("incorrect number of fields from pileup, did you forget the -c parameter?") if(@fields != 10);
	my ($chr, $pos, $ref, $cons, $cq, $sq, $rms, $d) = @fields[0..8];
	
	my $wi = pos2idx($pos);

	# Detect end of window condition
	if($chr ne $win_chr)
	{
		# Print the last window of the previous chr it this isnt the first
		printWindow() if $win_chr ne "-1";
		$win_chr = $chr;
		fill(0, $wi);
		initializeWindow($chr, $wi);
	}
	elsif($wi != $win_idx)
	{
		# Entered a new window, print last
		printWindow();
		fill($win_idx+1, $wi);
		initializeWindow($chr, $wi);
	}

	# Update data
	$s_depth += $d;
	$s_het++ if($cons !~ /[ACGTN]/);
	$s_rms += $rms;
	$num_points++;
	$num_covered++ if($d > 0);
}

# Print the last
printWindow();

sub initializeWindow
{
	my($c, $w) = @_;
	$win_chr = $c;
	$win_idx = $w;
	$win_start = $w * $win_size;
	$win_end = $win_start + $win_size;
	$s_depth = 0;
	$s_het = 0;
	$s_rms = 0;
	$num_points = 0;
	$num_covered = 0;
}

# Print the window in maq cns2win format
# Fields that are unused by cnD are left as 0
sub printWindow
{
	my $mp = ($win_start + ($win_size / 2)) / $scale;
	my $o_het = 0;
	my $o_depth = 0;
	my $o_rms = 0;
	if($num_points > 0)
	{
		$o_het = $s_het / $win_size;
		$o_depth = $s_depth / $win_size;
	}

	if($num_covered > 0)
	{
		my $exprms = $num_covered * 100;
		$o_rms = $s_rms / $exprms;
	}
	else
	{
		$o_rms = 1.0;
	}
	
	printf($formatStr, $win_chr, $mp, 0.0, $o_het, $o_depth, 0, $o_rms, 0);
}

sub fill
{
	my($from, $to) = @_;
	# Fill any gapped windows in between
	for(my $i = $from; $i < $to; ++$i)
	{
		initializeWindow($win_chr, $i);
		printWindow();
	}
}

sub pos2idx
{
	my($pos) = @_;
	return floor(($pos - 1) / $win_size);
}

