//-----------------------------------------------
// Copyright 2009 Wellcome Trust Sanger Institute
// Written by Jared Simpson (js18@sanger.ac.uk)
// Released under the GPL license
//-----------------------------------------------
import Alphabet;
import tango.util.collection.ArraySeq;
import tango.io.Stdout;
import tango.math.Math;


// Globals

// Column names
const char[] DEPTH_COL_NAME = "depth";
const char[] HET_COL_NAME = "het_rate";
const char[] AVGHIT_COL_NAME = "avghit";

char[][3] ALL_COLUMNS = [DEPTH_COL_NAME, HET_COL_NAME, AVGHIT_COL_NAME];

// State names

// Regular states
const char[] SNAME_ZERO = "Z";
const char[] SNAME_NORMAL = "N";
const char[] SNAME_HALF = "C0.5";
const char[] SNAME_C2 = "C2";
const char[] SNAME_C3 = "C3";

// Repeat states
const char[] REPEAT_PREFIX = "R";
const char[] SNAME_RPTZERO = REPEAT_PREFIX ~ SNAME_ZERO;
const char[] SNAME_RPTNORM = REPEAT_PREFIX ~ SNAME_NORMAL;
const char[] SNAME_RPTHALF = REPEAT_PREFIX ~ SNAME_HALF;
const char[] SNAME_RPTC2 = REPEAT_PREFIX ~ SNAME_C2;
const char[] SNAME_RPTC3 = REPEAT_PREFIX ~ SNAME_C3;
const char[] SNAME_RPTCMN = "RCOMMON";
const char[] SNAME_RPT = "R";

char[][1] REPEAT_STATES = [SNAME_RPT];
//char[][5] REPEAT_STATES = [SNAME_RPTZERO, SNAME_RPTNORM, SNAME_RPTHALF, SNAME_RPTC2, SNAME_RPTC3];
char[][6] REGULAR_STATES = [SNAME_ZERO, SNAME_NORMAL, SNAME_HALF, SNAME_C2, SNAME_C3, SNAME_RPT];

char[][2] GAIN_STATES = [SNAME_C2, SNAME_C3];
char[][2] LOSS_STATES = [SNAME_ZERO, SNAME_HALF];
char[][1] NORMAL_STATES = [SNAME_NORMAL];

// Data structure definitions
struct InputData
{
	char[] chr;
	real pos; // Coordinate in megabases
	real het_rate;
	real depth;
	real avg_hits; // average number of hits for reads falling in this range
}

//
// Simple collection of statistics
//
struct Statistics
{
	real mean;
	real median;
	real sd;
	int numDataPoints;
}

//
// The parameters a particular state can take
//
struct StateParameters
{
	real depth_mean;
	real depth_sd;
	real het_mean;
	real het_sd;
	real avghit_mean;
	real avghit_sd;

	char[] toString()
	{
		return Float.toString(depth_mean, 6) ~ "\t" ~
				Float.toString(depth_sd, 6) ~ "\t" ~
				Float.toString(het_mean, 6) ~ "\t" ~
				Float.toString(het_sd, 6) ~ "\t" ~
				Float.toString(avghit_mean, 6) ~ "\t" ~
				Float.toString(avghit_sd, 6);
	}
}

// Aliases
alias ArraySeq!(real) RealArray;
alias ArraySeq!(InputData) InputArray;
alias InputArray[char[]] StringInputArrayMap;
alias SymbolArray[char[]] StringSymbolArrayMap;


//
// Compute statistics from an array of values
//
Statistics computeStats(in real[] values)
{
	Statistics ret;
	
	values.sort;

	if(values.length == 0)
	{
		ret.mean = 0;
		ret.median = 0;
		ret.sd = 0;
		return ret;
	}

	real half_idx = cast(real)values.length / 2;
	if(values.length % 2 == 0)
	{
		// even number of elems
		ret.median = (values[rndint(half_idx)] + values[rndint(half_idx) - 1]) / 2;
	}
	else
	{
		ret.median = values[rndint(floor(half_idx))];
	}

	// Calculate the mean
	real sum = 0.0f;
	foreach(v; values)
	{
		sum += v;
	}
	ret.mean = sum / values.length;

	// Calculate the std dev
	sum = 0;
	foreach(v; values)
	{
		sum += pow((v - ret.mean), 2.0);
	}

	ret.sd = sqrt(sum / values.length);
	ret.numDataPoints = values.length;
	return ret;
}
