//-----------------------------------------------
// Copyright 2009 Wellcome Trust Sanger Institute
// Written by Jared Simpson (js18@sanger.ac.uk)
// Released under the GPL license
//-----------------------------------------------
import tango.math.Math;
import tango.io.Stdout;
import cnD;
import CNVModel;
import Common;
import Alphabet;
import Distribution;
import JointCNVDistribution;

class ModelParameters
{
	this(MarkovModel m)
	{
		// Initialize all the states
		StateParameters sp;
		foreach(s; m.getStateList())
		{
			stateParams[s.toString] = sp;
		}
	}

	char[] toString()
	{
		char[] outStr;
		foreach(l,s; stateParams)
		{
			outStr ~= l ~ "\t" ~ s.toString() ~ "\n";
		}
		return outStr;
	}

	void print()
	{
		foreach(l,s; stateParams)
		{
			Stdout.format("{}:\t\t{}\n", l, s.toString());
		}
	}

	//
	// Set the depth param for a particular state
	//
	void setDepth(char[] state, real mean, real sd)
	{
		stateParams[state].depth_mean = mean;
		stateParams[state].depth_sd = sd;
	}

	//
	// Set the het param for a particular state
	//
	void setHet(char[] state, real mean, real sd)
	{
		stateParams[state].het_mean = mean;
		stateParams[state].het_sd = sd;
	}

	//
	// Set the hit param for a particular state
	//
	void setRepeat(char[] state, real mean, real sd)
	{
		stateParams[state].avghit_mean = mean;
		stateParams[state].avghit_sd = sd;
	}

	//
	// Set depth parameters
	//
	void setDepthParameters(real normalMean, real normalSD, real zeroMean, real repeatMean, real repeatSD)
	{
		stateParams[SNAME_NORMAL].depth_mean = normalMean;
		stateParams[SNAME_NORMAL].depth_sd = normalSD;

		stateParams[SNAME_HALF].depth_mean = 0.5 * normalMean;
		stateParams[SNAME_HALF].depth_sd = 0.5 * normalSD;

		stateParams[SNAME_C2].depth_mean = 2 * normalMean;
		stateParams[SNAME_C2].depth_sd = 2 * normalSD;

		stateParams[SNAME_C3].depth_mean = 3 * normalMean;
		stateParams[SNAME_C3].depth_sd = 3 * normalSD;

		foreach(rn; REPEAT_STATES)
		{
			stateParams[rn].depth_mean = repeatMean;
			stateParams[rn].depth_sd = repeatSD;
		}

		stateParams[SNAME_ZERO].depth_mean = zeroMean;
		stateParams[SNAME_ZERO].depth_sd = zeroMean; //not used
	}

	//
	// Set repeat parameters
	//
	void setRepeatParameters(real unique_mean, real unique_sd, real zero_mean, real zero_sd, real repeat_mean, real repeat_sd)
	{
		stateParams[SNAME_ZERO].avghit_mean = zero_mean;
		stateParams[SNAME_ZERO].avghit_sd = zero_sd;

		stateParams[SNAME_NORMAL].avghit_mean = unique_mean;
		stateParams[SNAME_HALF].avghit_mean = unique_mean;
		stateParams[SNAME_C2].avghit_mean = unique_mean;
		stateParams[SNAME_C3].avghit_mean = unique_mean;

		stateParams[SNAME_NORMAL].avghit_sd = unique_sd;
		stateParams[SNAME_HALF].avghit_sd = unique_sd;
		stateParams[SNAME_C2].avghit_sd = unique_sd;
		stateParams[SNAME_C3].avghit_sd = unique_sd;

		foreach(rn; REPEAT_STATES)
		{
			stateParams[rn].avghit_mean = repeat_mean;
			stateParams[rn].avghit_sd = repeat_sd;
		}	
	}

	//
	// Set het params
	//
	void setHetParameters(real backgroundHet, real c2_h_mean, real c2_h_sd, real c3_h_mean, real c3_h_sd)
	{
		stateParams[SNAME_NORMAL].het_mean = backgroundHet;
		stateParams[SNAME_ZERO].het_mean = backgroundHet;
		stateParams[SNAME_HALF].het_mean = backgroundHet;
		stateParams[SNAME_C2].het_mean = c2_h_mean;
		stateParams[SNAME_C2].het_sd = c2_h_sd;

		stateParams[SNAME_C3].het_mean = c3_h_mean;
		stateParams[SNAME_C3].het_sd = c3_h_sd;

		foreach(rn; REPEAT_STATES)
		{
			stateParams[rn].het_mean = backgroundHet;
		}
	}
	
	// Per-state parameters
	StateParameters[char[]] stateParams;
}

