//-----------------------------------------------
// Copyright 2009 Wellcome Trust Sanger Institute
// Written by Jared Simpson (js18@sanger.ac.uk)
// Released under the GPL license
//-----------------------------------------------
module Alphabet;
import tango.io.Stdout;
import tango.text.convert.Integer;
import Float = tango.text.convert.Float;
import tango.util.collection.ArraySeq;

//
// Symbol and Alphabet implementations
// Interface inspired by the BioJava (biojava.org) project
//
class Symbol
{
	this() { m_name = ""; }
	this(char[] n) { m_name = n; }
	char[] toString() { return m_name; }
	char[] m_name;
}

class RealSymbol : Symbol
{
	this(char[] n, real v) { super(n); value = v; }
	char[] toString() { return super.toString() ~ " " ~ Float.toString(value); } 
	real value;
}

class IntSymbol : Symbol
{
	this(char[] n, int v) { super(n); value = v; }
	int value;

}

alias ArraySeq!(Symbol) SymbolArray;

//
// Alphabet interface
//
interface IAlphabet
{
	Symbol[] getSymbols();
}

//
// An alphabet is a collection of ordinary symbols
// It creates the symbols and maps the toString() property to a symbol object
//
class DiscreteAlphabet : IAlphabet
{
	Symbol[char[]] alphabet;
	
	Symbol createSymbol(char[] name)
	{
		Symbol s = new Symbol(name);

		// Ensure the symbol does not already exist
		if(contains(name))
		{
			throw new Exception("Attempt to add duplicate symbol: " ~ s.toString());
		}
		alphabet[s.toString()] = s;
		return s; // Reference to s
	}

	Symbol getSymbol(char[] name)
	{
		if(!contains(name))
		{
			throw new Exception("Symbol not found: " ~ name);
		}
		return *(name in alphabet);
	}

	char[][] getKeys()
	{
		return alphabet.keys;
	}

	Symbol[] getSymbols()
	{
		return alphabet.values;
	}

	// Check if the alphabet already contains a key with this value
	bool contains(char[] key)
	{
		Symbol* p = (key in alphabet);
		if(p == null)
		{
			return false;
		}
		return true;
	}


	// Foreach handler
	int opApply(int delegate(ref char[], ref Symbol) dg)
	{
		int result = 0;
		foreach(n,s; alphabet)
		{
			result = dg(n,s);
	   		if (result)
				break;
		}
		return result;			
	}
}


